local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local action = x.Action
local buff = x.Buff
local cooldown = x.Cooldown
local debuff = x.Debuff
local dot = x.Dot
local equipped = x.Equipped
local prev_gcd = x.PrevGCD
local talent = x.Talent
local target = x.Target


-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.EnemyDetected, x.EnemyDetected, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Example flash color coding:
-- White - Default
-- Yellow - Limited Time or No Global Cooldown
-- Purple - AOE or Positional Damage
-- Blue - AOE Debuff
-- Orange - Finishing Move
-- Aqua - Spell Interrupt, Reflect or Dispel
-- Green - Self Buff or Turn Autocast On
-- Red - Emergency Mitigation Cooldowns or Turn Autocast Off
-- Pink - Optional

a.spells["augmentation"] = {
	Type = "item",
	ID = 160053--[[Battle-Scarred Augment Rune]],
	Buff = 270058--[[Battle-Scarred Augmentation]],
	BuffUnit = "player",
	FlashColor = "Green",
	CheckFirst = function(z)
		return s.Boss("target")
	end,
}

a.spells["food"] = {
	Type = "item",
	ID = 156526--[[Bountiful Captain's Feast]],
	Buff = 257415--[[Well Fed]],
	BuffUnit = "player",
	FlashColor = "Green",
	CheckFirst = function(z)
		return s.Boss("target")
	end,
	FlashID = {
		156526--[[Bountiful Captain's Feast]],
		156525--[[Galley Banquet]],
		154884--[[Swamp Fish 'n Chips]],
		154883--[[Ravenberry Tarts]],
		154888--[[Sailor's Pie]],
		154887--[[Loa Loaf]]},
}

a.spells["flask"] = {
	Type = "item",
	ID = 152639--[[Flask of Endless Fathoms]],
	Buff = 251837--[[Flask of Endless Fathoms]],
	BuffUnit = "player",
	FlashColor = "Green",
	CheckFirst = function(z)
		return s.Boss("target")
	end,
}

a.spells["potion"] = {
	Type = "item",
	ID = 163222--[[Battle Potion of Intellect]],
	NoGCD = true,
	FlashColor = "Green",
	CheckFirst = function(z)
		return s.Boss("target")
	end,
}

a.spells["summon_pet"] = {
	ID = 712--[[Summon Succubus]],
	Check = function(z)
		return not UnitExists("pet") and x.ShouldPermanentBuff
	end,
	FlashColor = "Green",
	FlashID = {712--[[Summon Succubus]], 691--[[Summon Felhunter]], 697--[[Summon Voidwalker]], 688--[[Summon Imp]], 30146--[[Summon Felguard]]},
}

,"variable,name=use_seed,value=talent.sow_the_seeds.enabled&spell_targets.seed_of_corruption_aoe>=3|talent.siphon_life.enabled&spell_targets.seed_of_corruption>=5|spell_targets.seed_of_corruption>=8"
,"variable,name=padding,op=set,value=action.shadow_bolt.execute_time*azerite.cascading_calamity.enabled"
,"variable,name=padding,op=reset,value=gcd,if=azerite.cascading_calamity.enabled&(talent.drain_soul.enabled|talent.deathbolt.enabled&cooldown.deathbolt.remains<=gcd)"
,"call_action_list,name=cooldowns"
,"drain_soul,interrupt_global=1,chain=1,cycle_targets=1,if=target.time_to_die<=gcd&soul_shard<5"
,"haunt,if=spell_targets.seed_of_corruption_aoe<=2"
,"summon_darkglare,if=dot.agony.ticking&dot.corruption.ticking&(buff.active_uas.stack=5|soul_shard=0)&(!talent.phantom_singularity.enabled|cooldown.phantom_singularity.remains)"
,"agony,target_if=min:dot.agony.remains,if=remains<=gcd+action.shadow_bolt.execute_time&target.time_to_die>8"
,"unstable_affliction,target_if=!contagion&target.time_to_die<=8"
,"drain_soul,target_if=min:debuff.shadow_embrace.remains,interrupt_immediate=1,interrupt_if=ticks_remain<5,if=talent.shadow_embrace.enabled&active_enemies<=2&debuff.shadow_embrace.remains&debuff.shadow_embrace.remains<=gcd*2"
,"shadow_bolt,target_if=min:debuff.shadow_embrace.remains,if=talent.shadow_embrace.enabled&talent.absolute_corruption.enabled&active_enemies<=2&debuff.shadow_embrace.remains&debuff.shadow_embrace.remains<=execute_time*2+travel_time&!action.shadow_bolt.in_flight"
,"phantom_singularity,target_if=max:target.time_to_die,if=time>35&(cooldown.summon_darkglare.remains>=45|cooldown.summon_darkglare.remains<8)&target.time_to_die>16*spell_haste"
,"vile_taint,target_if=max:target.time_to_die,if=time>15&target.time_to_die>=10"
,"unstable_affliction,if=!variable.use_seed&soul_shard=5"
,"seed_of_corruption,if=variable.use_seed&soul_shard=5"
,"call_action_list,name=dots"
,"phantom_singularity,if=time<=35"
,"vile_taint,if=time<15"
,"dark_soul"
,"berserking"
,"call_action_list,name=spenders"
,"call_action_list,name=fillers"

-- Cooldowns
,"potion,if=(talent.dark_soul_misery.enabled&cooldown.summon_darkglare.up&cooldown.dark_soul.up)|cooldown.summon_darkglare.up|target.time_to_die<30"
,"use_items,if=!cooldown.summon_darkglare.up"
,"fireblood,if=!cooldown.summon_darkglare.up"
,"blood_fury,if=!cooldown.summon_darkglare.up"

-- Dots
,"seed_of_corruption,if=dot.corruption.remains<=action.seed_of_corruption.cast_time+time_to_shard+4.2*(1-talent.creeping_death.enabled*0.15)&spell_targets.seed_of_corruption_aoe>=3+talent.writhe_in_agony.enabled&!dot.seed_of_corruption.remains&!action.seed_of_corruption.in_flight"
,"agony,target_if=min:remains,if=talent.creeping_death.enabled&active_dot.agony<6&target.time_to_die>10&(remains<=gcd|cooldown.summon_darkglare.remains>10&refreshable)"
,"agony,target_if=min:remains,if=!talent.creeping_death.enabled&active_dot.agony<8&target.time_to_die>10&(remains<=gcd|cooldown.summon_darkglare.remains>10&refreshable)"
,"siphon_life,target_if=min:remains,if=-spell_targets.sow_the_seeds_aoe<5&target.time_to_die>10&refreshable&!(cooldown.summon_darkglare.remains<=soul_shard*action.unstable_affliction.execute_time)"
,"siphon_life,target_if=min:remains,if=(active_dot.siphon_life<8-talent.creeping_death.enabled-spell_targets.sow_the_seeds_aoe)&target.time_to_die>10&refreshable&!(cooldown.summon_darkglare.remains<=soul_shard*action.unstable_affliction.execute_time)"
,"corruption,cycle_targets=1,if=active_enemies<3+talent.writhe_in_agony.enabled&(remains<=gcd|cooldown.summon_darkglare.remains>10&refreshable)&target.time_to_die>10"

-- Fillers
,"agony,if=talent.deathbolt.enabled&cooldown.summon_darkglare.remains>=30+gcd&cooldown.deathbolt.remains<=gcd&!prev_gcd.1.summon_darkglare&!prev_gcd.1.agony&talent.writhe_in_agony.enabled&azerite.sudden_onset.enabled&remains<duration*0.5"
,"deathbolt,if=cooldown.summon_darkglare.remains>=30+gcd|cooldown.summon_darkglare.remains>140"
,"shadow_bolt,if=buff.movement.up&buff.nightfall.remains"
,"agony,if=buff.movement.up&!(talent.siphon_life.enabled&(prev_gcd.1.agony&prev_gcd.2.agony&prev_gcd.3.agony)|prev_gcd.1.agony)"
,"siphon_life,if=buff.movement.up&!(prev_gcd.1.siphon_life&prev_gcd.2.siphon_life&prev_gcd.3.siphon_life)"
,"corruption,if=buff.movement.up&!prev_gcd.1.corruption&!talent.absolute_corruption.enabled"
,"drain_life,if=(buff.inevitable_demise.stack>=90&(cooldown.deathbolt.remains>execute_time|!talent.deathbolt.enabled)&(cooldown.phantom_singularity.remains>execute_time|!talent.phantom_singularity.enabled)&(cooldown.dark_soul.remains>execute_time|!talent.dark_soul_misery.enabled)&(cooldown.vile_taint.remains>execute_time|!talent.vile_taint.enabled)&cooldown.summon_darkglare.remains>execute_time+10|buff.inevitable_demise.stack>30&target.time_to_die<=10)"
,"haunt"
,"drain_soul,interrupt_global=1,chain=1,interrupt=1,cycle_targets=1,if=target.time_to_die<=gcd"
,"drain_soul,target_if=min:debuff.shadow_embrace.remains,chain=1,interrupt_if=ticks_remain<5,interrupt_global=1,if=talent.shadow_embrace.enabled&active_enemies=2&!debuff.shadow_embrace.remains"
,"drain_soul,target_if=min:debuff.shadow_embrace.remains,chain=1,interrupt_if=ticks_remain<5,interrupt_global=1,if=talent.shadow_embrace.enabled&active_enemies=2"
,"drain_soul,interrupt_global=1,chain=1,interrupt=1"
,"shadow_bolt,cycle_targets=1,if=talent.shadow_embrace.enabled&talent.absolute_corruption.enabled&active_enemies=2&!debuff.shadow_embrace.remains&!action.shadow_bolt.in_flight"
,"shadow_bolt,target_if=min:debuff.shadow_embrace.remains,if=talent.shadow_embrace.enabled&talent.absolute_corruption.enabled&active_enemies=2"
,"shadow_bolt"

-- Spenders
,"unstable_affliction,if=cooldown.summon_darkglare.remains<=soul_shard*execute_time"
,"call_action_list,name=fillers,if=(cooldown.summon_darkglare.remains<time_to_shard*(5-soul_shard)|cooldown.summon_darkglare.up)&time_to_die>cooldown.summon_darkglare.remains"
,"seed_of_corruption,if=variable.use_seed"
,"unstable_affliction,if=!variable.use_seed&!prev_gcd.1.summon_darkglare&(talent.deathbolt.enabled&cooldown.deathbolt.remains<=execute_time&!azerite.cascading_calamity.enabled|soul_shard>=2&target.time_to_die>4+execute_time&active_enemies=1|target.time_to_die<=8+execute_time*soul_shard)"
,"unstable_affliction,if=!variable.use_seed&contagion<=cast_time+variable.padding"
,"unstable_affliction,cycle_targets=1,if=!variable.use_seed&(!talent.deathbolt.enabled|cooldown.deathbolt.remains>time_to_shard|soul_shard>1)&contagion<=cast_time+variable.padding&(!azerite.cascading_calamity.enabled|buff.cascading_calamity.remains>time_to_shard)"
