local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local action = x.Action
local buff = x.Buff
local cooldown = x.Cooldown
local debuff = x.Debuff
local dot = x.Dot
local equipped = x.Equipped
local prev_gcd = x.PrevGCD
local talent = x.Talent
local target = x.Target


-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.EnemyDetected, x.EnemyDetected, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Example flash color coding:
-- White - Default
-- Yellow - Limited Time or No Global Cooldown
-- Purple - AOE or Positional Damage
-- Blue - AOE Debuff
-- Orange - Finishing Move
-- Aqua - Spell Interrupt, Reflect or Dispel
-- Green - Self Buff or Turn Autocast On
-- Red - Emergency Mitigation Cooldowns or Turn Autocast Off
-- Pink - Optional

a.spells["demonbolt"] = {
	ID = 264178--[[Demonbolt]],
	Check = function(z)
		return not x.InCombat
	end,
}

-- Executed every time the actor is available.
a.spells["potion,if=pet.demonic_tyrant.active|target.time_to_die<30"] = {
	Type = "item",
	ID = 163222--[[Battle Potion of Intellect]],
	NoGCD = true,
	FlashColor = "Yellow",
	CheckFirst = function(z)
		return s.Boss("target") and (buff.demonic_power.up_nogcd or target.time_to_die < 30)
	end,
}

,"use_items,if=pet.demonic_tyrant.active|target.time_to_die<=15"

a.spells["berserking,if=pet.demonic_tyrant.active|target.time_to_die<=15"] = {
	ID = 26297--[[Berserking]],
	NoGCD = true,
	FlashColor = "Yellow",
	CheckFirst = function(z)
		return x.UseLongCD and (buff.demonic_power.up_nogcd or target.time_to_die <= 15)
	end,
}

a.spells["blood_fury,if=pet.demonic_tyrant.active|target.time_to_die<=15"] = {
	ID = 33702--[[Blood Fury]],
	NoGCD = true,
	FlashColor = "Yellow",
	CheckFirst = function(z)
		return x.UseLongCD and (buff.demonic_power.up_nogcd or target.time_to_die <= 15)
	end,
}

a.spells["fireblood,if=pet.demonic_tyrant.active|target.time_to_die<=15"] = {
	ID = 265221--[[Fireblood]],
	NoGCD = true,
	FlashColor = "Yellow",
	CheckFirst = function(z)
		return x.UseLongCD and (buff.demonic_power.up_nogcd or target.time_to_die <= 15)
	end,
}

a.spells["doom,if=!ticking&time_to_die>30&spell_targets.implosion<2"] = {
	ID = 265412--[[Doom]],
	Check = function(z)
		return not dot.doom.ticking and target.time_to_die > 30 and x.ActiveEnemies < 2
	end,
}

a.spells["demonic_strength,if=(buff.wild_imps.stack<6|buff.demonic_power.up)|spell_targets.implosion<2"] = {
	ID = 267171--[[Demonic Strength]],
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.PetActiveEnemy and ((buff.wild_imps.stack < 6 or buff.demonic_power.up) or x.ActiveEnemies < 2)
	end,
}

a.spells["grimoire_felguard,if=cooldown.summon_demonic_tyrant.remains<13|!equipped.132369"] = {
	ID = 111898--[[Grimoire: Felguard]],
	NoRangeCheck = true,
	EnemyTargetNeeded = true,
	Check = function(z)
		return  x.UseLongCD and (cooldown.summon_demonic_tyrant.remains < 13 or not equipped[132369])
	end,
}

a.spells["summon_vilefiend,if=equipped.132369|cooldown.summon_demonic_tyrant.remains>40|cooldown.summon_demonic_tyrant.remains<12"] = {
	ID = 264119--[[Summon Vilefiend]],
	Check = function(z)
		return equipped[132369] or cooldown.summon_demonic_tyrant.remains > 40 or cooldown.summon_demonic_tyrant.remains < 12 or not x.UseLongCD
	end,
}

a.spells["call_dreadstalkers,if=equipped.132369|(cooldown.summon_demonic_tyrant.remains<9&buff.demonic_calling.remains)|(cooldown.summon_demonic_tyrant.remains<11&!buff.demonic_calling.remains)|cooldown.summon_demonic_tyrant.remains>14"] = {
	ID = 104316--[[Call Dreadstalkers]],
	NotWhileMoving = true,
	Check = function(z)
		return not s.Casting(104316--[[Call Dreadstalkers]], "player") and (equipped[132369] or ((cooldown.summon_demonic_tyrant.remains < 9 or not x.UseLongCD) and buff.demonic_calling.remains) or ((cooldown.summon_demonic_tyrant.remains < 11 or not x.UseLongCD) and not buff.demonic_calling.up) or cooldown.summon_demonic_tyrant.remains > 14 or not x.UseLongCD)
	end,
}


a.spells["summon_demonic_tyrant,if=equipped.132369|(buff.dreadstalkers.remains>cast_time&(buff.wild_imps.stack>=3|prev_gcd.1.hand_of_guldan)&(soul_shard<3|buff.dreadstalkers.remains<gcd*2.7|buff.grimoire_felguard.remains<gcd*2.7))"] = {
	ID = 265187--[[Summon Demonic Tyrant]],
	NoRangeCheck = true,
	Check = function(z)
		return x.UseLongCD and (equipped[132369] or (buff.dreadstalkers.remains > action.summon_demonic_tyrant.cast_time and (buff.wild_imps.stack >= 3 or prev_gcd[1].hand_of_guldan) and (x.SoulShards < 3 or buff.dreadstalkers.remains < x.MaxGCD * 2.7 or buff.grimoire_felguard.remains < x.MaxGCD * 2.7)))
	end,
}

a.spells["power_siphon,if=buff.wild_imps.stack>=2&buff.demonic_core.stack<=2&buff.demonic_power.down&spell_targets.implosion<2"] = {
	ID = 264130--[[Power Siphon]],
	NoRangeCheck = true,
	Check = function(z)
		return buff.wild_imps.stack >= 2 and buff.demonic_core.stack <= 2 and buff.demonic_power.down and x.ActiveEnemies < 2
	end,
}

a.spells["doom,if=talent.doom.enabled&refreshable&time_to_die>(dot.doom.remains+30)"] = {
	ID = 265412--[[Doom]],
	Check = function(z)
		return talent.doom.enabled and dsot.doom.refreshable and target.time_to_die > (dot.doom.remains + 30)
	end,
}

a.spells["hand_of_guldan,if=soul_shard>=5|(soul_shard>=3&cooldown.call_dreadstalkers.remains>4&(!talent.summon_vilefiend.enabled|cooldown.summon_vilefiend.remains>3))"] = {
	ID = 105174--[[Hand of Gul'dan]],
	NotWhileMoving = true,
	Check = function(z)
		return x.SoulShards >= 5 or (x.SoulShards >= 3 and cooldown.call_dreadstalkers.remains > 4 and (not talent.summon_vilefiend.enabled or cooldown.summon_vilefiend.remains > 3))
	end,
}

a.spells["soul_strike,if=soul_shard<5&buff.demonic_core.stack<=2"] = {
	ID = 264057--[[Soul Strike]],
	Check = function(z)
		return x.PetAlive and x.SoulShards < 5 and buff.demonic_core.stack <= 2
	end,
}

a.spells["demonbolt,if=soul_shard<=3&buff.demonic_core.up&((cooldown.summon_demonic_tyrant.remains<10|cooldown.summon_demonic_tyrant.remains>22)|buff.demonic_core.stack>=3|buff.demonic_core.remains<5|time_to_die<25)"] = {
	ID = 264178--[[Demonbolt]],
	Check = function(z)
		return x.SoulShards <= 3 and buff.demonic_core.up and ((cooldown.summon_demonic_tyrant.remains < 10 or cooldown.summon_demonic_tyrant.remains > 22 or not x.UseLongCD) or buff.demonic_core.stack >= 3 or buff.demonic_core.remains < 5 or target.time_to_die < 25)
	end,
}

--[[
  ___                 _           _             
 |_ _|_ __ ___  _ __ | | ___  ___(_) ___  _ __  
  | || '_ ` _ \| '_ \| |/ _ \/ __| |/ _ \| '_ \ 
  | || | | | | | |_) | | (_) \__ \ | (_) | | | |
 |___|_| |_| |_| .__/|_|\___/|___/_|\___/|_| |_|
               |_|                              
]]
a.spells["implosion,if=(buff.wild_imps.stack>=6&(soul_shard<3|prev_gcd.1.call_dreadstalkers|buff.wild_imps.stack>=9|prev_gcd.1.bilescourge_bombers|(!prev_gcd.1.hand_of_guldan&!prev_gcd.2.hand_of_guldan))&!prev_gcd.1.hand_of_guldan&!prev_gcd.2.hand_of_guldan&buff.demonic_power.down)|(time_to_die<3&buff.wild_imps.stack>0)|(prev_gcd.2.call_dreadstalkers&buff.wild_imps.stack>2&!talent.demonic_calling.enabled)"] = {
	ID = 196277--[[Implosion]],
	Check = function(z)
		return (buff.wild_imps.stack >= 6 and (x.SoulShards < 3 or prev_gcd[1].call_dreadstalkers or buff.wild_imps.stack >= 9 or prev_gcd[1].bilescourge_bombers or (not prev_gcd[1].hand_of_guldan and not prev_gcd[2].hand_of_guldan)) and not prev_gcd[1].hand_of_guldan and not prev_gcd[2].hand_of_guldan and buff.demonic_power.down) or (target.time_to_die < 3 and buff.wild_imps.stack > 0) or (prev_gcd[2].call_dreadstalkers and buff.wild_imps.stack > 2 and not talent.demonic_calling.enabled)
	end,
}

a.spells["call_dreadstalkers,if=(cooldown.summon_demonic_tyrant.remains<9&buff.demonic_calling.remains)|(cooldown.summon_demonic_tyrant.remains<11&!buff.demonic_calling.remains)|cooldown.summon_demonic_tyrant.remains>14"] = {
	ID = 104316--[[Call Dreadstalkers]],
	NotWhileMoving = true,
	Check = function(z)
		return not s.Casting(104316--[[Call Dreadstalkers]], "player") and (((cooldown.summon_demonic_tyrant.remains < 9 or not x.UseLongCD) and buff.demonic_calling.remains) or ((cooldown.summon_demonic_tyrant.remains < 11 or not x.UseLongCD) and not buff.demonic_calling.up) or cooldown.summon_demonic_tyrant.remains > 14 or not x.UseLongCD)
	end,
}

a.spells["summon_demonic_tyrant"] = {
	ID = 265187--[[Summon Demonic Tyrant]],
	NoRangeCheck = true,
	Check = function(z)
		return x.UseLongCD
	end,
}

a.spells["hand_of_guldan,if=soul_shard>=3&(((prev_gcd.2.hand_of_guldan|buff.wild_imps.stack>=3)&buff.wild_imps.stack<9)|cooldown.summon_demonic_tyrant.remains<=gcd*2|buff.demonic_power.remains>gcd*2)"] = {
	ID = 105174--[[Hand of Gul'dan]],
	NotWhileMoving = true,
	Check = function(z)
		return x.SoulShards >= 3 and (((prev_gcd[2].hand_of_guldan or buff.wild_imps.stack >= 3) and buff.wild_imps.stack < 9) or cooldown.summon_demonic_tyrant.remains <= x.MaxGCD * 2  or not x.UseLongCD or buff.demonic_power.remains > x.MaxGCD * 2)
	end,
}

a.spells["demonbolt,if=prev_gcd.1.hand_of_guldan&soul_shard>=1&(buff.wild_imps.stack<=3|prev_gcd.3.hand_of_guldan)&soul_shard<4&buff.demonic_core.up"] = {
	ID = 264178--[[Demonbolt]],
	Check = function(z)
		return prev_gcd[1].hand_of_guldan and x.SoulShards >= 1 and (buff.wild_imps.stack<=3 or prev_gcd[3].hand_of_guldan) and x.SoulShards < 4 and buff.demonic_core.up
	end,
}

a.spells["summon_vilefiend,if=(cooldown.summon_demonic_tyrant.remains>40&spell_targets.implosion<=2)|cooldown.summon_demonic_tyrant.remains<12"] = {
	ID = 264119--[[Summon Vilefiend]],
	Check = function(z)
		return (cooldown.summon_demonic_tyrant.remains > 40 and x.ActiveEnemies <= 2) or cooldown.summon_demonic_tyrant.remains < 12 or not x.UseLongCD
	end,
}

a.spells["bilescourge_bombers,if=cooldown.summon_demonic_tyrant.remains>9"] = {
	ID = 267211--[[Bilescourge Bombers]],
	NoRangeCheck = true,
	Check = function(z)
		return cooldown.summon_demonic_tyrant.remains > 9 or not x.UseLongCD
	end,
}

a.spells["soul_strike,if=soul_shard<5&buff.demonic_core.stack<=2"] = {
	ID = 264057--[[Soul Strike]],
	Check = function(z)
		return x.PetAlive and x.SoulShards < 5 and buff.demonic_core.stack <= 2
	end,
}

a.spells["demonbolt,if=soul_shard<=3&buff.demonic_core.up&(buff.demonic_core.stack>=3|buff.demonic_core.remains<=gcd*5.7)"] = {
	ID = 264178--[[Demonbolt]],
	Check = function(z)
		return x.SoulShards <= 3 and buff.demonic_core.up and (buff.demonic_core.stack >= 3 or buff.demonic_core.remains <= x.MaxGCD * 5.7)
	end,
}

a.spells["doom,cycle_targets=1,max_cycle_targets=7,if=refreshable"] = {
	ID = 265412--[[Doom]],
	Check = function(z)
		return dot.doom.refreshable
	end,
}

--[[
  _   _      _   _                 ____            _        _      _        _   _           
 | \ | | ___| |_| |__   ___ _ __  |  _ \ ___  _ __| |_ __ _| |    / \   ___| |_(_)_   _____ 
 |  \| |/ _ \ __| '_ \ / _ \ '__| | |_) / _ \| '__| __/ _` | |   / _ \ / __| __| \ \ / / _ \
 | |\  |  __/ |_| | | |  __/ |    |  __/ (_) | |  | || (_| | |  / ___ \ (__| |_| |\ V /  __/
 |_| \_|\___|\__|_| |_|\___|_|    |_|   \___/|_|   \__\__,_|_| /_/   \_\___|\__|_| \_/ \___|
                                                                                            
]]

a.spells["summon_vilefiend,if=cooldown.summon_demonic_tyrant.remains>40|cooldown.summon_demonic_tyrant.remains<12"] = {
	ID = 264119--[[Summon Vilefiend]],
	Check = function(z)
		return cooldown.summon_demonic_tyrant.remains > 40 or cooldown.summon_demonic_tyrant.remains < 12 or not x.UseLongCD
	end,
}

a.spells["hand_of_guldan,if=((cooldown.call_dreadstalkers.remains>action.demonbolt.cast_time)&(cooldown.call_dreadstalkers.remains>action.shadow_bolt.cast_time))&cooldown.nether_portal.remains>(160+action.hand_of_guldan.cast_time)"] = {
	ID = 105174--[[Hand of Gul'dan]],
	NotWhileMoving = true,
	Check = function(z)
		return ((cooldown.call_dreadstalkers.remains > action.demonbolt.cast_time) and (cooldown.call_dreadstalkers.remains > action.shadow_bolt.cast_time)) and cooldown.nether_portal.remains > (160 + action.hand_of_guldan.cast_time)
	end,
}

a.spells["summon_demonic_tyrant,if=buff.nether_portal.remains<10&soul_shard=0"] = {
	ID = 265187--[[Summon Demonic Tyrant]],
	NoRangeCheck = true,
	Check = function(z)
		return x.UseLongCD and buff.nether_portal.remains < 10 and x.SoulShards == 0
	end,
}

a.spells["summon_demonic_tyrant,if=buff.nether_portal.remains<action.summon_demonic_tyrant.cast_time+5.5"] = {
	ID = 265187--[[Summon Demonic Tyrant]],
	NoRangeCheck = true,
	Check = function(z)
		return x.UseLongCD and buff.nether_portal.remains < action.summon_demonic_tyrant.cast_time + 5.5
	end,
}

a.spells["demonbolt,if=buff.demonic_core.up"] = {
	ID = 264178--[[Demonbolt]],
	Check = function(z)
		return buff.demonic_core.up
	end,
}

--[[
  _   _      _   _                 ____            _        _   ____        _ _     _ _             
 | \ | | ___| |_| |__   ___ _ __  |  _ \ ___  _ __| |_ __ _| | | __ ) _   _(_) | __| (_)_ __   __ _ 
 |  \| |/ _ \ __| '_ \ / _ \ '__| | |_) / _ \| '__| __/ _` | | |  _ \| | | | | |/ _` | | '_ \ / _` |
 | |\  |  __/ |_| | | |  __/ |    |  __/ (_) | |  | || (_| | | | |_) | |_| | | | (_| | | | | | (_| |
 |_| \_|\___|\__|_| |_|\___|_|    |_|   \___/|_|   \__\__,_|_| |____/ \__,_|_|_|\__,_|_|_| |_|\__, |
                                                                                              |___/ 
]]
a.spells["nether_portal,if=soul_shard>=5&(!talent.power_siphon.enabled|buff.demonic_core.up)"] = {
	ID = 267217--[[Nether Portal]],
	NotWhileMoving = true,
	Check = function(z)
		return x.SoulShards >= 5 and (not talent.power_siphon.enabled or buff.demonic_core.up)
	end,
}

a.spells["call_dreadstalkers"] = {
	ID = 104316--[[Call Dreadstalkers]],
	NotWhileMoving = true,
	Check = function(z)
		return not s.Casting(104316--[[Call Dreadstalkers]], "player")
	end,
}

a.spells["hand_of_guldan,if=cooldown.call_dreadstalkers.remains>18&soul_shard>=3"] = {
	ID = 105174--[[Hand of Gul'dan]],
	NotWhileMoving = true,
	Check = function(z)
		return cooldown.call_dreadstalkers.remains > 18 and x.SoulShards >= 3
	end,
}

a.spells["power_siphon,if=buff.wild_imps.stack>=2&buff.demonic_core.stack<=2&buff.demonic_power.down&soul_shard>=3"] = {
	ID = 264130--[[Power Siphon]],
	NoRangeCheck = true,
	Check = function(z)
		return buff.wild_imps.stack >= 2 and buff.demonic_core.stack <= 2 and buff.demonic_power.down and x.SoulShards >= 3
	end,
}

a.spells["hand_of_guldan,if=soul_shard>=5"] = {
	ID = 105174--[[Hand of Gul'dan]],
	NotWhileMoving = true,
	Check = function(z)
		return x.SoulShards >= 5
	end,
}

--[[
  ____        _ _     _           ____  _                   _ 
 | __ ) _   _(_) | __| |   __ _  / ___|| |__   __ _ _ __ __| |
 |  _ \| | | | | |/ _` |  / _` | \___ \| '_ \ / _` | '__/ _` |
 | |_) | |_| | | | (_| | | (_| |  ___) | | | | (_| | | | (_| |
 |____/ \__,_|_|_|\__,_|  \__,_| |____/|_| |_|\__,_|_|  \__,_|
                                                              
]]

a.spells["demonbolt,if=azerite.forbidden_knowledge.enabled&buff.forbidden_knowledge.react&!buff.demonic_core.react&cooldown.summon_demonic_tyrant.remains>20"] = {
	ID = 264178--[[Demonbolt]],
	Check = function(z)
		return buff.forbidden_knowledge.up and not buff.demonic_core.up and (cooldown.summon_demonic_tyrant.remains > 20 or not x.UseLongCD)
	end,
}

a.spells["soul_strike"] = {
	ID = 264057--[[Soul Strike]],
	Check = function(z)
		return x.PetAlive
	end,
}

a.spells["shadow_bolt"] = {
	ID = 686--[[Shadow Bolt]],
	Check = function(z)
		return true
	end,
}

a.BuildAShard = function()
	s.UpdateBuff(279666, "Forbidden Knowledge")
	return a.Flash(
		"demonbolt,if=azerite.forbidden_knowledge.enabled&buff.forbidden_knowledge.react&!buff.demonic_core.react&cooldown.summon_demonic_tyrant.remains>20"
		,"soul_strike"
		,"shadow_bolt"
	)
end
